/*
 * Copyright 2017 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.kie.workbench.common.stunner.core.graph.content.view;

import org.jboss.errai.common.client.api.annotations.MapsTo;
import org.jboss.errai.common.client.api.annotations.NonPortable;
import org.jboss.errai.common.client.api.annotations.Portable;
import org.kie.workbench.common.stunner.core.util.HashUtil;

@Portable
public class MagnetImpl implements Magnet {

    private Point2D location;
    private MagnetType magnetType;

    private MagnetImpl(final @MapsTo("location") Point2D location,
                       final @MapsTo("magnetType") MagnetType magnetType) {
        this.location = location;
        this.magnetType = magnetType;
    }

    public void setLocation(final Point2D location) {
        this.location = location;
    }

    public void setMagnetType(final MagnetType magnetType) {
        this.magnetType = magnetType;
    }

    @Override
    public Point2D getLocation() {
        return location;
    }

    @Override
    public MagnetType getMagnetType() {
        return magnetType;
    }

    @Override
    public String toString() {
        return "[MagnetImpl at {" + location + "}, magnetType = " + magnetType + "]";
    }

    @Override
    public boolean equals(Object o) {
        if (o instanceof MagnetImpl) {
            MagnetImpl other = (MagnetImpl) o;
            return ((null != location)?
                    (location.getX() == other.location.getX() && location.getY() == other.location.getY()) :
                    null == other.location) &&
                    ((null != magnetType)? magnetType.equals(other.magnetType) : null == other.magnetType);
        }
        return false;
    }

    @Override
    public int hashCode() {
        return HashUtil.combineHashCodes((null != location)? HashUtil.combineHashCodes(1,Double.hashCode(location.getX()),
                                                                                       Double.hashCode(location.getY()))
                                                 : 0,
                                         (null != magnetType)? magnetType.hashCode() : 0);
    }

    @NonPortable
    public static class Builder {

        public static MagnetImpl build(final double x,
                                       final double y) {
            return new MagnetImpl(new Point2D(x,
                                              y),
                                  null);
        }

        public static MagnetImpl build(final MagnetType magnetType) {
            return new MagnetImpl(null,
                                  magnetType);
        }
    }
}
